﻿using Microsoft.EntityFrameworkCore;
using System;

namespace Benday.EfCore.Api
{
    public class MyDbContext : DbContext
    {
        public DbSet<Person> Persons { get; set; }
        public DbSet<Media> Media { get; set; }
        public DbSet<Book> Books { get; set; }
        public DbSet<Movie> Movies { get; set; }
        public DbSet<MusicAlbum> MusicAlbums { get; set; }

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            optionsBuilder.UseSqlServer(@"Server=(local);Database=EfCoreEnterpriseSample;Trusted_Connection=True;");            
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Phone>(entity =>
            {
                entity.ToTable("PersonPhone");

                entity.Property(e => e.PhoneDescription).IsRequired();
                entity.Property(e => e.PhoneNumber).IsRequired();

                entity.HasOne(d => d.Owner)
                    .WithMany(p => p.PhoneNumbers)
                    .HasForeignKey(d => d.OwnerId)
                    .OnDelete(Microsoft.EntityFrameworkCore.Metadata.DeleteBehavior.Cascade)
                    .HasConstraintName("FK_Phone_Person");
            });

            modelBuilder.Entity<Person>()
                .ToTable("Person")
                .Property(x => x.LastUpdated)
                .HasDefaultValueSql("CONVERT(date, '1/1/1900')")
                .ValueGeneratedOnAddOrUpdate()                
                ;

            modelBuilder.Entity<Media>()
                .HasDiscriminator(e => e.MediaType)
                .HasValue<Book>("Book")
                .HasValue<Movie>("Movie")
                .HasValue<MusicAlbum>("Music Album");


        }
    }
}
