﻿using System;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata;

namespace Scaffold123
{
    public partial class ScaffoldDbContext : DbContext
    {
        public virtual DbSet<Animal> Animal { get; set; }
        public virtual DbSet<AnimalFood> AnimalFood { get; set; }
        public virtual DbSet<Company> Company { get; set; }
        public virtual DbSet<Employee> Employee { get; set; }
        public virtual DbSet<EmployeePhone> EmployeePhone { get; set; }
        public virtual DbSet<Food> Food { get; set; }

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            optionsBuilder.UseSqlServer(@"Server=(local); Database=Scaffold123; Trusted_Connection=true;");
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Animal>(entity =>
            {
                entity.Property(e => e.Name).IsRequired();
            });

            modelBuilder.Entity<AnimalFood>(entity =>
            {
                entity.HasKey(e => new { e.AnimalId, e.FoodId })
                    .HasName("PK_AnimalFood");

                entity.HasOne(d => d.Animal)
                    .WithMany(p => p.AnimalFood)
                    .HasForeignKey(d => d.AnimalId)
                    .OnDelete(DeleteBehavior.Cascade)
                    .HasConstraintName("FK_AnimalFood_Animal");

                entity.HasOne(d => d.Food)
                    .WithMany(p => p.AnimalFood)
                    .HasForeignKey(d => d.FoodId)
                    .OnDelete(DeleteBehavior.Restrict)
                    .HasConstraintName("FK_AnimalFood_Food");
            });

            modelBuilder.Entity<Company>(entity =>
            {
                entity.Property(e => e.Name).IsRequired();
            });

            modelBuilder.Entity<Employee>(entity =>
            {
                entity.Property(e => e.FirstName)
                    .IsRequired()
                    .HasMaxLength(50);

                entity.Property(e => e.LastName)
                    .IsRequired()
                    .HasMaxLength(50);

                entity.HasOne(d => d.Company)
                    .WithMany(p => p.Employee)
                    .HasForeignKey(d => d.CompanyId)
                    .OnDelete(DeleteBehavior.Restrict)
                    .HasConstraintName("FK_Employee_Company");
            });

            modelBuilder.Entity<EmployeePhone>(entity =>
            {
                entity.Property(e => e.PhoneNumber)
                    .IsRequired()
                    .HasMaxLength(50);

                entity.Property(e => e.PhoneType)
                    .IsRequired()
                    .HasMaxLength(50);

                entity.HasOne(d => d.Employee)
                    .WithMany(p => p.EmployeePhone)
                    .HasForeignKey(d => d.EmployeeId)
                    .OnDelete(DeleteBehavior.Restrict)
                    .HasConstraintName("FK_EmployeePhone_Employee");
            });

            modelBuilder.Entity<Food>(entity =>
            {
                entity.Property(e => e.Name).IsRequired();
            });
        }
    }
}