﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Scaffold123;
using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;

namespace Benday.EfCore.Tests.Scaffold123
{
    [TestClass]
    public class AnimalFixture
    {
        
        [TestMethod]
        public void CreateAnimalsAndFoods()
        {
            using (var data = new AnimalTestDataManager())
            {
                data.CreateAnimalsAndFoodsAndSave();
            }
        }

        [TestMethod]
        public void CreateAnimalsAndFoodsAndAssignFoods()
        {
            using (var data = new AnimalTestDataManager())
            {
                data.CreateAnimalsAndFoodsAndSave();
                
                data.AddFoods();

                Animal animal = data.GetAnimalByName(
                    AnimalTestDataManager.AnimalHumanOmnivore);
                Assert.IsNotNull(animal, "Could not find animal.");
                Assert.AreNotEqual<int>(0, animal.AnimalFood.Count, "Food count was wrong.");

                data.Context.SaveChanges();
            }
        }

        [TestMethod]
        public void GetFoodsUsingStoredProcedure()
        {
            Animal animal = null;

            using (var data = new AnimalTestDataManager())
            {
                data.CreateAnimalsAndFoodsAndSave();

                data.AddFoods();
                animal = data.GetAnimalByName(
                                    AnimalTestDataManager.AnimalHumanOmnivore);

                data.Context.SaveChanges();
            }

            var expected = animal.AnimalFood.Select(x => x.Food).ToList();
            
            var service = new MiscellaneousManagementService();
            var actual = service.GetFoodsByAnimalId(animal.Id);

            Assert.AreNotEqual<int>(0, actual.Count, "Should have gotten at least one food back.");
            Assert.AreEqual<int>(expected.Count, actual.Count, "Food count was wrong.");
        }

        [TestMethod]
        public void DeleteAnimalFoodFromAnimalCascadesTheDeleteButDoesNotDeleteFood()
        {
            AnimalFood animalFood = null;

            using (var data = new AnimalTestDataManager())
            {
                data.CreateAnimalsAndFoodsAndSave();
                data.AddFoods();
                data.Context.SaveChanges();

                var animal = data.Animals[1];
                animalFood = animal.AnimalFood.ToList()[1];
                var food = animalFood.Food;

                animal.AnimalFood.Remove(animalFood);

                data.Context.SaveChanges();
            }

            using (var context = new ScaffoldDbContext())
            {
                var matchAnimalFood = (from temp in context.AnimalFood
                             where temp.AnimalId == animalFood.AnimalId &&
                             temp.FoodId == animalFood.FoodId
                             select temp).FirstOrDefault();

                Assert.IsNull(matchAnimalFood, 
                    "Value for animal id '{0}' and food id '{1}' was not deleted.", 
                    animalFood.AnimalId, animalFood.FoodId);

                var matchFood = (from temp in context.Food
                                       where temp.Id == animalFood.FoodId
                                       select temp).FirstOrDefault();

                Assert.IsNotNull(matchFood,
                    "Food id '{0}' was deleted.  It wasn't supposed to be deleted.",
                    animalFood.FoodId);
            }
        }
    }
}
