﻿using Scaffold123;
using System.Collections.Generic;
using System;
using System.Linq;

namespace Benday.EfCore.Tests.Scaffold123
{
    public class AnimalTestDataManager : IDisposable
    {
        public const string AnimalHumanOmnivore = "Human (Omnivore)";
        public const string AnimalHumanVegetarian = "Human (Vegetarian)";
        public const string AnimalCat = "Cat";
        public const string FoodChocolate = "Chocolate";
        public const string FoodChicken = "Chicken";
        public const string FoodBeef = "Beef";
        public const string FoodOranges = "Oranges";
        public const string FoodSteamedRice = "Steamed Rice";
        public const string FoodTofu = "Tofu";
        public const string FoodDonuts = "Donuts";
        public const string FoodCatFood = "Cat Food";


        public AnimalTestDataManager()
        {

        }
        public List<Animal> Animals { get; set; }
        public List<Food> Foods { get; set; }
        public ScaffoldDbContext Context { get; set; }
        
        public void AddFoods()
        {
            var data = this;

            AddFood(data, AnimalHumanOmnivore, FoodChocolate);
            AddFood(data, AnimalHumanOmnivore, FoodBeef);
            AddFood(data, AnimalHumanOmnivore, FoodChicken);

            AddFood(data, AnimalHumanVegetarian, FoodTofu);
            AddFood(data, AnimalHumanVegetarian, FoodSteamedRice);
            AddFood(data, AnimalHumanVegetarian, FoodOranges);

            AddFood(data, AnimalCat, FoodDonuts);
            AddFood(data, AnimalCat, FoodCatFood);
        }

        private void AddFood(
            AnimalTestDataManager data,
            string animalType,
            string foodType)
        {
            Animal animal = GetAnimalByName(data, animalType);

            if (animal == null)
            {
                throw new InvalidOperationException("Could not find animal.");
            }

            var food = (from temp in data.Foods
                        where temp.Name == foodType
                        select temp).FirstOrDefault();

            if (food == null)
            {
                throw new InvalidOperationException("Could not find food.");
            }

            animal.AddFood(food);
        }

        private Animal GetAnimalByName(AnimalTestDataManager data, string animalType)
        {
            return (from temp in data.Animals
                    where temp.Name == animalType
                    select temp).FirstOrDefault();
        }

        public Animal GetAnimalByName(string animalType)
        {
            return GetAnimalByName(this, animalType);
        }


        public void CreateAnimalsAndFoodsAndSave()
        {
            var humanOmnivore = new Animal()
            {
                Name = AnimalTestDataManager.AnimalHumanOmnivore
            };

            var humanVegetarian = new Animal()
            {
                Name = AnimalTestDataManager.AnimalHumanVegetarian
            };

            var cat = new Animal()
            {
                Name = AnimalTestDataManager.AnimalCat
            };

            var animals = new List<Animal>();

            animals.Add(humanOmnivore);
            animals.Add(humanVegetarian);
            animals.Add(cat);

            var foods = CreateFoods();
            
            var data = this;

            data.Animals = animals;
            data.Foods = foods;

            var context = new ScaffoldDbContext();

            data.Context = context;

            context.Animal.AddRange(animals);
            context.Food.AddRange(foods);

            context.SaveChanges();
        }

        public List<Food> CreateFoods()
        {
            List<Food> returnValues = new List<Food>();

            returnValues.Add(CreateFood(FoodChocolate));
            returnValues.Add(CreateFood(FoodChicken));
            returnValues.Add(CreateFood(FoodBeef));
            returnValues.Add(CreateFood(FoodOranges));
            returnValues.Add(CreateFood(FoodSteamedRice));
            returnValues.Add(CreateFood(FoodTofu));
            returnValues.Add(CreateFood(FoodDonuts));
            returnValues.Add(CreateFood(FoodCatFood));

            return returnValues;
        }

        private Food CreateFood(string name)
        {
            return new Food() { Name = name };
        }

        #region IDisposable Support
        private bool disposedValue = false; // To detect redundant calls

        protected virtual void Dispose(bool disposing)
        {
            if (!disposedValue)
            {
                if (disposing)
                {
                    // TODO: dispose managed state (managed objects).
                }

                if (Context != null)
                {
                    Context.Dispose();
                    Context = null;
                }

                disposedValue = true;
            }
        }

        // TODO: override a finalizer only if Dispose(bool disposing) above has code to free unmanaged resources.
        // ~AnimalTestDataManager() {
        //   // Do not change this code. Put cleanup code in Dispose(bool disposing) above.
        //   Dispose(false);
        // }

        // This code added to correctly implement the disposable pattern.
        public void Dispose()
        {
            // Do not change this code. Put cleanup code in Dispose(bool disposing) above.
            Dispose(true);
            // TODO: uncomment the following line if the finalizer is overridden above.
            // GC.SuppressFinalize(this);
        }
        #endregion
    }
}
